@@ -19,10 +19,13 @@
  * 02110-1301, USA.
  */
 
+#define _GNU_SOURCE // To access vasprintf
+
 #include <assert.h>
 #include <regex.h>
 #include <getopt.h>
 #include <errno.h>
+#include "version.h"
 #include "config.h"
 #include "misc.h"
 #include "print.h"
@@ -1085,7 +1088,7 @@ void options_parse(int argc, char *argv[])
                 break;
 
             case 'v':
-                printf("tio v%s\n", VERSION);
+                printf("tio %s\n", VERSION);
                 exit(EXIT_SUCCESS);
                 break;
 
