@@ -22,6 +22,7 @@
 #if defined(__linux__)
 #include <linux/serial.h>
 #endif
+#include "version.h"
 #include "config.h"
 #include <stdarg.h>
 #include <stdio.h>
@@ -1090,7 +1091,7 @@ void handle_command_sequence(char input_char, char *output_char, bool *forward)
                 break;
 
             case KEY_V:
-                tio_printf("tio v%s", VERSION);
+                tio_printf("tio %s", VERSION);
                 break;
 
             case KEY_X:
@@ -1614,6 +1615,7 @@ const char* get_serial_port_type(const char* port_name)
 
 const char* get_serial_port_type(const char* port_name)
 {
+    (void)port_name;
     return "";
 }
 
@@ -1735,7 +1737,8 @@ GList *tty_search_for_serial_devices(void)
 
         // Hash remaining string to get unique topology ID
         unsigned long hash = djb2_hash((const unsigned char *)devices_path);
-        char *tid = base62_encode(hash);
+        char tid[5];
+        base62_encode(hash, tid);
         free(devices_path);
 
         // Construct the path to the device's driver symlink
